@@ -319,10 +319,10 @@ schedule_write_to_child_python(unsigned long id, const char *prefix, PyObject *a
 static PyObject *
 needs_write(ChildMonitor UNUSED *self, PyObject *args) {
 #define needs_write_doc "needs_write(id, data) -> Queue data to be written to child."
-    unsigned long id, sz;
-    const char *data;
-    if (!PyArg_ParseTuple(args, "ks#", &id, &data, &sz)) return NULL;
-    if (schedule_write_to_child(id, 1, data, (size_t)sz)) { Py_RETURN_TRUE; }
+    unsigned long id;
+    Py_buffer buf;
+    if (!PyArg_ParseTuple(args, "ky*", &id, &buf)) return NULL;
+    if (schedule_write_to_child(id, 1, buf.buf, (size_t)buf.len)) { Py_RETURN_TRUE; }
     Py_RETURN_FALSE;
 }
 
@@ -553,8 +553,6 @@ pyset_iutf8(ChildMonitor *self, PyObject *args) {
 #undef INCREF_CHILD
 #undef DECREF_CHILD
 
-extern void cocoa_update_menu_bar_title(PyObject*);
-
 static bool
 cursor_needs_render(Window *w) {
     return w->cursor_visible_at_last_render != w->render_data.screen->cursor_render_info.is_visible || w->last_cursor_x != w->render_data.screen->cursor_render_info.x || w->last_cursor_y != w->render_data.screen->cursor_render_info.y || w->last_cursor_shape != w->render_data.screen->cursor_render_info.shape;
@@ -592,7 +590,7 @@ change_menubar_title(PyObject *title UNUSED) {
     static PyObject *current_title = NULL;
     if (title != current_title) {
         current_title = title;
-        if (title && OPT(macos_show_window_title_in) & MENUBAR) cocoa_update_menu_bar_title(title);
+        if (title && OPT(macos_show_window_title_in) & MENUBAR) update_menu_bar_title(title);
     }
 #endif
 }
@@ -639,6 +637,7 @@ prepare_to_render_os_window(OSWindow *os_window, monotonic_t now, unsigned int *
             if (is_active_window) {
                 *active_window_id = w->id;
                 if (collect_cursor_info(&WD.screen->cursor_render_info, w, now, os_window)) needs_render = true;
+                WD.screen->cursor_render_info.is_focused = os_window->is_focused;
                 set_os_window_title_from_window(w, os_window);
                 *active_window_bg = window_bg;
             } else {
@@ -1001,21 +1000,21 @@ static bool cocoa_pending_actions[NUM_COCOA_PENDING_ACTIONS] = {0};
 static bool has_cocoa_pending_actions = false;
 typedef struct {
     char* wd;
-    char **open_files;
-    size_t open_files_count;
-    size_t open_files_capacity;
+    char **open_urls;
+    size_t open_urls_count;
+    size_t open_urls_capacity;
 } CocoaPendingActionsData;
 static CocoaPendingActionsData cocoa_pending_actions_data = {0};
 
 void
-set_cocoa_pending_action(CocoaPendingAction action, const char *wd) {
-    if (wd) {
-        if (action == OPEN_FILE) {
-            ensure_space_for(&cocoa_pending_actions_data, open_files, char*, cocoa_pending_actions_data.open_files_count + 8, open_files_capacity, 8, true);
-            cocoa_pending_actions_data.open_files[cocoa_pending_actions_data.open_files_count++] = strdup(wd);
+set_cocoa_pending_action(CocoaPendingAction action, const char *data) {
+    if (data) {
+        if (action == LAUNCH_URLS) {
+            ensure_space_for(&cocoa_pending_actions_data, open_urls, char*, cocoa_pending_actions_data.open_urls_count + 8, open_urls_capacity, 8, true);
+            cocoa_pending_actions_data.open_urls[cocoa_pending_actions_data.open_urls_count++] = strdup(data);
         } else {
             if (cocoa_pending_actions_data.wd) free(cocoa_pending_actions_data.wd);
-            cocoa_pending_actions_data.wd = strdup(wd);
+            cocoa_pending_actions_data.wd = strdup(data);
         }
     }
     cocoa_pending_actions[action] = true;
@@ -1038,23 +1037,25 @@ process_cocoa_pending_actions(void) {
     if (cocoa_pending_actions[NEW_WINDOW]) { call_boss(new_window, NULL); }
     if (cocoa_pending_actions[CLOSE_WINDOW]) { call_boss(close_window, NULL); }
     if (cocoa_pending_actions[RESET_TERMINAL]) { call_boss(clear_terminal, "sO", "reset", Py_True ); }
+    if (cocoa_pending_actions[CLEAR_TERMINAL_AND_SCROLLBACK]) { call_boss(clear_terminal, "sO", "to_cursor", Py_True ); }
     if (cocoa_pending_actions[RELOAD_CONFIG]) { call_boss(load_config_file, NULL); }
     if (cocoa_pending_actions[TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY]) { call_boss(toggle_macos_secure_keyboard_entry, NULL); }
+    if (cocoa_pending_actions[TOGGLE_FULLSCREEN]) { call_boss(toggle_fullscreen, NULL); }
     if (cocoa_pending_actions_data.wd) {
-        if (cocoa_pending_actions[NEW_OS_WINDOW_WITH_WD]) { call_boss(new_os_window_with_wd, "s", cocoa_pending_actions_data.wd); }
-        if (cocoa_pending_actions[NEW_TAB_WITH_WD]) { call_boss(new_tab_with_wd, "s", cocoa_pending_actions_data.wd); }
+        if (cocoa_pending_actions[NEW_OS_WINDOW_WITH_WD]) { call_boss(new_os_window_with_wd, "sO", cocoa_pending_actions_data.wd, Py_True); }
+        if (cocoa_pending_actions[NEW_TAB_WITH_WD]) { call_boss(new_tab_with_wd, "sO", cocoa_pending_actions_data.wd, Py_True); }
         free(cocoa_pending_actions_data.wd);
         cocoa_pending_actions_data.wd = NULL;
     }
-    if (cocoa_pending_actions_data.open_files_count) {
-        for (unsigned cpa = 0; cpa < cocoa_pending_actions_data.open_files_count; cpa++) {
-            if (cocoa_pending_actions_data.open_files[cpa]) {
-                call_boss(open_file, "s", cocoa_pending_actions_data.open_files[cpa]);
-                free(cocoa_pending_actions_data.open_files[cpa]);
-                cocoa_pending_actions_data.open_files[cpa] = NULL;
+    if (cocoa_pending_actions_data.open_urls_count) {
+        for (unsigned cpa = 0; cpa < cocoa_pending_actions_data.open_urls_count; cpa++) {
+            if (cocoa_pending_actions_data.open_urls[cpa]) {
+                call_boss(launch_urls, "s", cocoa_pending_actions_data.open_urls[cpa]);
+                free(cocoa_pending_actions_data.open_urls[cpa]);
+                cocoa_pending_actions_data.open_urls[cpa] = NULL;
             }
         }
-        cocoa_pending_actions_data.open_files_count = 0;
+        cocoa_pending_actions_data.open_urls_count = 0;
     }
     memset(cocoa_pending_actions, 0, sizeof(cocoa_pending_actions));
     has_cocoa_pending_actions = false;
@@ -1114,11 +1115,11 @@ main_loop(ChildMonitor *self, PyObject *a UNUSED) {
     run_main_loop(process_global_state, self);
 #ifdef __APPLE__
     if (cocoa_pending_actions_data.wd) { free(cocoa_pending_actions_data.wd); cocoa_pending_actions_data.wd = NULL; }
-    if (cocoa_pending_actions_data.open_files) {
-        for (unsigned cpa = 0; cpa < cocoa_pending_actions_data.open_files_count; cpa++) {
-            if (cocoa_pending_actions_data.open_files[cpa]) free(cocoa_pending_actions_data.open_files[cpa]);
+    if (cocoa_pending_actions_data.open_urls) {
+        for (unsigned cpa = 0; cpa < cocoa_pending_actions_data.open_urls_count; cpa++) {
+            if (cocoa_pending_actions_data.open_urls[cpa]) free(cocoa_pending_actions_data.open_urls[cpa]);
         }
-        free(cocoa_pending_actions_data.open_files); cocoa_pending_actions_data.open_files = NULL;
+        free(cocoa_pending_actions_data.open_urls); cocoa_pending_actions_data.open_urls = NULL;
     }
 #endif
     if (PyErr_Occurred()) return NULL;
