@@ -231,8 +231,10 @@ PENDING(previous_tab, PREVIOUS_TAB)
 PENDING(new_window, NEW_WINDOW)
 PENDING(close_window, CLOSE_WINDOW)
 PENDING(reset_terminal, RESET_TERMINAL)
+PENDING(clear_terminal_and_scrollback, CLEAR_TERMINAL_AND_SCROLLBACK)
 PENDING(reload_config, RELOAD_CONFIG)
 PENDING(toggle_macos_secure_keyboard_entry, TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY)
+PENDING(toggle_fullscreen, TOGGLE_FULLSCREEN)
 
 - (void)open_kitty_website_url:(id)sender {
     (void)sender;
@@ -242,6 +244,8 @@ PENDING(toggle_macos_secure_keyboard_entry, TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY)
 - (BOOL)validateMenuItem:(NSMenuItem *)item {
     if (item.action == @selector(toggle_macos_secure_keyboard_entry:)) {
         item.state = [SecureKeyboardEntryController sharedInstance].isDesired ? NSControlStateValueOn : NSControlStateValueOff;
+    } else if (item.action == @selector(toggle_fullscreen:)) {
+        item.title = ([NSApp currentSystemPresentationOptions] & NSApplicationPresentationFullScreen) ? @"Exit Full Screen" : @"Enter Full Screen";
     }
     return YES;
 }
@@ -270,8 +274,8 @@ typedef struct {
 } GlobalShortcut;
 typedef struct {
     GlobalShortcut new_os_window, close_os_window, close_tab, edit_config_file, reload_config;
-    GlobalShortcut previous_tab, next_tab, new_tab, new_window, close_window, reset_terminal;
-    GlobalShortcut toggle_macos_secure_keyboard_entry;
+    GlobalShortcut previous_tab, next_tab, new_tab, new_window, close_window, reset_terminal, clear_terminal_and_scrollback;
+    GlobalShortcut toggle_macos_secure_keyboard_entry, toggle_fullscreen;
 } GlobalShortcuts;
 static GlobalShortcuts global_shortcuts;
 
@@ -285,8 +289,8 @@ cocoa_set_global_shortcut(PyObject *self UNUSED, PyObject *args) {
 #define Q(x) if (strcmp(name, #x) == 0) gs = &global_shortcuts.x
     Q(new_os_window); else Q(close_os_window); else Q(close_tab); else Q(edit_config_file);
     else Q(new_tab); else Q(next_tab); else Q(previous_tab);
-    else Q(new_window); else Q(close_window); else Q(reset_terminal); else Q(reload_config);
-    else Q(toggle_macos_secure_keyboard_entry);
+    else Q(new_window); else Q(close_window); else Q(reset_terminal); else Q(clear_terminal_and_scrollback); else Q(reload_config);
+    else Q(toggle_macos_secure_keyboard_entry); else Q(toggle_fullscreen);
 #undef Q
     if (gs == NULL) { PyErr_SetString(PyExc_KeyError, "Unknown shortcut name"); return NULL; }
     int cocoa_mods;
@@ -495,25 +499,41 @@ cocoa_send_notification(PyObject *self UNUSED, PyObject *args) {
 
 - (BOOL)openTab:(NSPasteboard*)pasteboard
         userData:(NSString *) UNUSED userData error:(NSError **) UNUSED error {
-    return [self openFilesFromPasteboard:pasteboard type:NEW_TAB_WITH_WD];
+    return [self openDirsFromPasteboard:pasteboard type:NEW_TAB_WITH_WD];
 }
 
 - (BOOL)openOSWindow:(NSPasteboard*)pasteboard
         userData:(NSString *) UNUSED userData  error:(NSError **) UNUSED error {
-    return [self openFilesFromPasteboard:pasteboard type:NEW_OS_WINDOW_WITH_WD];
+    return [self openDirsFromPasteboard:pasteboard type:NEW_OS_WINDOW_WITH_WD];
 }
 
-- (BOOL)openFilesFromPasteboard:(NSPasteboard *)pasteboard type:(int)type {
+- (BOOL)openDirsFromPasteboard:(NSPasteboard *)pasteboard type:(int)type {
     NSDictionary *options = @{ NSPasteboardURLReadingFileURLsOnlyKey: @YES };
     NSArray *filePathArray = [pasteboard readObjectsForClasses:[NSArray arrayWithObject:[NSURL class]] options:options];
+    NSMutableArray<NSString*> *dirPathArray = [NSMutableArray arrayWithCapacity:[filePathArray count]];
     for (NSURL *url in filePathArray) {
         NSString *path = [url path];
         BOOL isDirectory = NO;
         if ([[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&isDirectory]) {
-            if (!isDirectory) {
-                path = [path stringByDeletingLastPathComponent];
-            }
-            set_cocoa_pending_action(type, [path UTF8String]);
+            if (!isDirectory) path = [path stringByDeletingLastPathComponent];
+            if (![dirPathArray containsObject:path]) [dirPathArray addObject:path];
+        }
+    }
+    if ([dirPathArray count] > 0) {
+        // Colons are not valid in paths under macOS.
+        set_cocoa_pending_action(type, [[dirPathArray componentsJoinedByString:@":"] UTF8String]);
+    }
+    return YES;
+}
+
+- (BOOL)openFileURLs:(NSPasteboard*)pasteboard
+        userData:(NSString *) UNUSED userData  error:(NSError **) UNUSED error {
+    NSDictionary *options = @{ NSPasteboardURLReadingFileURLsOnlyKey: @YES };
+    NSArray *urlArray = [pasteboard readObjectsForClasses:[NSArray arrayWithObject:[NSURL class]] options:options];
+    for (NSURL *url in urlArray) {
+        NSString *path = [url path];
+        if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
+            set_cocoa_pending_action(LAUNCH_URLS, [[[NSURL fileURLWithPath:path] absoluteString] UTF8String]);
         }
     }
     return YES;
@@ -593,6 +613,7 @@ cocoa_create_global_menu(void) {
     MENU_ITEM(shellMenu, @"Close Window", close_window);
     [shellMenu addItem:[NSMenuItem separatorItem]];
     MENU_ITEM(shellMenu, @"Reset", reset_terminal);
+    MENU_ITEM(shellMenu, @"Clear to Cursor Line", clear_terminal_and_scrollback);
     [shellMenu release];
 
     NSMenuItem* windowMenuItem =
@@ -621,10 +642,7 @@ cocoa_create_global_menu(void) {
                     keyEquivalent:@""] setTarget:global_menu_target];
 
     [windowMenu addItem:[NSMenuItem separatorItem]];
-    [[windowMenu addItemWithTitle:@"Enter Full Screen"
-                           action:@selector(toggleFullScreen:)
-                    keyEquivalent:@"f"]
-     setKeyEquivalentModifierMask:NSEventModifierFlagControl | NSEventModifierFlagCommand];
+    MENU_ITEM(windowMenu, @"Enter Full Screen", toggle_fullscreen);
     [NSApp setWindowsMenu:windowMenu];
     [windowMenu release];
 
@@ -656,7 +674,16 @@ cocoa_create_global_menu(void) {
 
 void
 cocoa_update_menu_bar_title(PyObject *pytitle) {
-    NSString *title = @(PyUnicode_AsUTF8(pytitle));
+    NSString *title = nil;
+    if (OPT(macos_menubar_title_max_length) > 0 && PyUnicode_GetLength(pytitle) > OPT(macos_menubar_title_max_length)) {
+        static char fmt[64];
+        snprintf(fmt, sizeof(fmt), "%%%ld.%ldU%%s", OPT(macos_menubar_title_max_length), OPT(macos_menubar_title_max_length));
+        DECREF_AFTER_FUNCTION PyObject *st = PyUnicode_FromFormat(fmt, pytitle, "…");
+        if (st) title = @(PyUnicode_AsUTF8(st));
+    } else {
+        title = @(PyUnicode_AsUTF8(pytitle));
+    }
+    if (!title) return;
     NSMenu *bar = [NSApp mainMenu];
     if (title_menu != NULL) {
         [bar removeItem:title_menu];
@@ -814,6 +841,33 @@ cocoa_set_titlebar_color(void *w, color_type titlebar_color)
     } // autoreleasepool
 }
 
+static PyObject*
+cocoa_set_url_handler(PyObject UNUSED *self, PyObject *args) {
+    @autoreleasepool {
+
+    const char *url_scheme = NULL, *bundle_id = NULL;
+    if (!PyArg_ParseTuple(args, "s|z", &url_scheme, &bundle_id)) return NULL;
+    if (!url_scheme || url_scheme[0] == '\0') {
+        PyErr_SetString(PyExc_TypeError, "Empty url scheme");
+        return NULL;
+    }
+
+    NSString *scheme = [NSString stringWithUTF8String:url_scheme];
+    NSString *identifier = @"";
+    if (!bundle_id) {
+        identifier = [[NSBundle mainBundle] bundleIdentifier];
+        if (!identifier || identifier.length == 0) identifier = @"net.kovidgoyal.kitty";
+    } else if (bundle_id[0] != '\0') {
+        identifier = [NSString stringWithUTF8String:bundle_id];
+    }
+    // This API has been marked as deprecated. It will need to be replaced when a new approach is available.
+    OSStatus err = LSSetDefaultHandlerForURLScheme((CFStringRef)scheme, (CFStringRef)identifier);
+    if (err == noErr) Py_RETURN_NONE;
+    PyErr_Format(PyExc_OSError, "Failed to set default handler with error code: %d", err);
+    return NULL;
+    } // autoreleasepool
+}
+
 static NSSound *beep_sound = nil;
 
 static void
@@ -863,6 +917,7 @@ static PyMethodDef module_methods[] = {
     {"cocoa_set_global_shortcut", (PyCFunction)cocoa_set_global_shortcut, METH_VARARGS, ""},
     {"cocoa_send_notification", (PyCFunction)cocoa_send_notification, METH_VARARGS, ""},
     {"cocoa_set_notification_activated_callback", (PyCFunction)set_notification_activated_callback, METH_O, ""},
+    {"cocoa_set_url_handler", (PyCFunction)cocoa_set_url_handler, METH_VARARGS, ""},
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
