@@ -67,11 +67,11 @@ static char mouse_event_buf[64];
 
 static int
 encode_mouse_event_impl(const MousePosition *mpos, int mouse_tracking_protocol, int button, MouseAction action, int mods) {
-    unsigned int cb = 0;
+    unsigned int cb = encode_button(button);
     if (action == MOVE) {
-        cb = 3;
+        if (cb == UINT_MAX) cb = 3;
+        cb += 32;
     } else {
-        cb = encode_button(button);
         if (cb == UINT_MAX) return 0;
     }
     if (action == DRAG || action == MOVE) cb |= MOTION_INDICATOR;
@@ -141,6 +141,13 @@ static bool
 dispatch_mouse_event(Window *w, int button, int count, int modifiers, bool grabbed) {
     bool handled = false;
     if (w->render_data.screen && w->render_data.screen->callbacks != Py_None) {
+        PyObject *callback_ret = PyObject_CallMethod(w->render_data.screen->callbacks, "on_mouse_event", "{si si si sO}",
+            "button", button, "repeat_count", count, "mods", modifiers, "grabbed", grabbed ? Py_True : Py_False);
+        if (callback_ret == NULL) PyErr_Print();
+        else {
+            handled = callback_ret == Py_True;
+            Py_DECREF(callback_ret);
+        }
         if (OPT(debug_keyboard)) {
             const char *evname = "move";
             switch(count) {
@@ -162,14 +169,7 @@ dispatch_mouse_event(Window *w, int button, int count, int modifiers, bool grabb
                 case GLFW_MOUSE_BUTTON_7: bname = "b7"; break;
                 case GLFW_MOUSE_BUTTON_8: bname = "b8"; break;
             }
-            debug("\x1b[33mon_mouse_input\x1b[m: %s button: %s %sgrabbed: %d\n", evname, bname, format_mods(modifiers), grabbed);
-        }
-        PyObject *callback_ret = PyObject_CallMethod(w->render_data.screen->callbacks, "on_mouse_event", "{si si si sO}",
-            "button", button, "repeat_count", count, "mods", modifiers, "grabbed", grabbed ? Py_True : Py_False);
-        if (callback_ret == NULL) PyErr_Print();
-        else {
-            handled = callback_ret == Py_True;
-            Py_DECREF(callback_ret);
+            debug("\x1b[33mon_mouse_input\x1b[m: %s button: %s %sgrabbed: %d handled_in_kitty: %d\n", evname, bname, format_mods(modifiers), grabbed, handled);
         }
     }
     return handled;
@@ -377,18 +377,22 @@ clear_click_queue(Window *w, int button) {
 
 #define N(n) (q->clicks[q->length - n])
 
+static double
+radius_for_multiclick(void) {
+    return 0.5 * (global_state.callback_os_window ? global_state.callback_os_window->fonts_data->cell_height : 8);
+}
+
 static bool
-release_is_click(Window *w, int button) {
-    ClickQueue *q = &w->click_queues[button];
-    double click_allowed_radius = 1.2 * (global_state.callback_os_window ? global_state.callback_os_window->fonts_data->cell_height : 20);
+release_is_click(const Window *w, int button) {
+    const ClickQueue *q = &w->click_queues[button];
     monotonic_t now = monotonic();
-    return (q->length > 0 && distance(N(1).x, N(1).y, MAX(0, w->mouse_pos.global_x), MAX(0, w->mouse_pos.global_y)) <= click_allowed_radius && now - N(1).at < OPT(click_interval));
+    return (q->length > 0 && distance(N(1).x, N(1).y, MAX(0, w->mouse_pos.global_x), MAX(0, w->mouse_pos.global_y)) <= radius_for_multiclick() && now - N(1).at < OPT(click_interval));
 }
 
 static unsigned
-multi_click_count(Window *w, int button) {
-    ClickQueue *q = &w->click_queues[button];
-    double multi_click_allowed_radius = 1.2 * (global_state.callback_os_window ? global_state.callback_os_window->fonts_data->cell_height : 20);
+multi_click_count(const Window *w, int button) {
+    const ClickQueue *q = &w->click_queues[button];
+    double multi_click_allowed_radius = radius_for_multiclick();
     if (q->length > 2) {
         // possible triple-click
         if (
@@ -414,7 +418,8 @@ add_press(Window *w, int button, int modifiers) {
     ClickQueue *q = &w->click_queues[button];
     if (q->length == CLICK_QUEUE_SZ) { memmove(q->clicks, q->clicks + 1, sizeof(Click) * (CLICK_QUEUE_SZ - 1)); q->length--; }
     monotonic_t now = monotonic();
-    N(0).at = now; N(0).button = button; N(0).modifiers = modifiers; N(0).x = MAX(0, w->mouse_pos.global_x); N(0).y = MAX(0, w->mouse_pos.global_y);
+    static unsigned long num = 0;
+    N(0).at = now; N(0).button = button; N(0).modifiers = modifiers; N(0).x = MAX(0, w->mouse_pos.global_x); N(0).y = MAX(0, w->mouse_pos.global_y); N(0).num = ++num;
     q->length++;
     Screen *screen = w->render_data.screen;
     int count = multi_click_count(w, button);
@@ -459,6 +464,8 @@ typedef struct PendingClick {
     bool grabbed;
     monotonic_t at;
     MousePosition mouse_pos;
+    unsigned long press_num;
+    double radius_for_multiclick;
 } PendingClick;
 
 static void
@@ -467,14 +474,24 @@ free_pending_click(id_type timer_id UNUSED, void *pc) { free(pc); }
 void
 send_pending_click_to_window(Window *w, void *data) {
     PendingClick *pc = (PendingClick*)data;
-    ClickQueue *q = &w->click_queues[pc->button];
-    // only send click if no presses have happened since the release that triggered the click
-    if (q->length && q->clicks[q->length - 1].at <= pc->at) {
+    const ClickQueue *q = &w->click_queues[pc->button];
+    // only send click if no presses have happened since the release that triggered
+    // the click or if the subsequent press is too far or too late for a double click
+    if (!q->length) return;
+#define press(n) q->clicks[q->length - n]
+    if (
+            press(1).at <= pc->at || // latest press is before click release
+            (q->length > 1 && press(2).num == pc->press_num &&  (   // penultimate press is the press that belongs to this click
+                press(1).at - press(2).at > OPT(click_interval) ||  // too long between the presses for it to be a double click
+                distance(press(1).x, press(1).y, press(2).x, press(2).y) > pc->radius_for_multiclick  // presses are too far apart
+            ))
+    ) {
         MousePosition current_pos = w->mouse_pos;
         w->mouse_pos = pc->mouse_pos;
         dispatch_mouse_event(w, pc->button, pc->count, pc->modifiers, pc->grabbed);
         w->mouse_pos = current_pos;
     }
+#undef press
 }
 
 static void
@@ -484,6 +501,8 @@ dispatch_possible_click(Window *w, int button, int modifiers) {
     if (release_is_click(w, button)) {
         PendingClick *pc = calloc(sizeof(PendingClick), 1);
         if (pc) {
+            const ClickQueue *q = &w->click_queues[button];
+            pc->press_num = q->length ? q->clicks[q->length - 1].num : 0;
             pc->window_id = w->id;
             pc->mouse_pos = w->mouse_pos;
             pc->at = monotonic();
@@ -491,6 +510,7 @@ dispatch_possible_click(Window *w, int button, int modifiers) {
             pc->count = count == 2 ? -3 : -2;
             pc->modifiers = modifiers;
             pc->grabbed = screen->modes.mouse_tracking_mode != 0;
+            pc->radius_for_multiclick = radius_for_multiclick();
             add_main_loop_timer(OPT(click_interval), false, send_pending_click_to_window_id, pc, free_pending_click);
         }
     }
@@ -701,7 +721,10 @@ mouse_event(const int button, int modifiers, int action) {
     bool in_tab_bar;
     unsigned int window_idx = 0;
     Window *w = NULL;
-    debug("%s mouse_button: %d %s", action == GLFW_RELEASE ? "\x1b[32mRelease\x1b[m" : (button < 0 ? "\x1b[36mMove\x1b[m" : "\x1b[31mPress\x1b[m"), button, format_mods(modifiers));
+    if (OPT(debug_keyboard)) {
+        if (button < 0) { debug("%s x: %.1f y: %.1f ", "\x1b[36mMove\x1b[m", global_state.callback_os_window->mouse_x, global_state.callback_os_window->mouse_y); }
+        else { debug("%s mouse_button: %d %s", action == GLFW_RELEASE ? "\x1b[32mRelease\x1b[m" : "\x1b[31mPress\x1b[m", button, format_mods(modifiers)); }
+    }
     if (global_state.redirect_mouse_handling) {
         w = window_for_event(&window_idx, &in_tab_bar);
         call_boss(mouse_event, "OK iiii dd",
@@ -760,9 +783,9 @@ mouse_event(const int button, int modifiers, int action) {
                 clamp_to_window = true;
                 Tab *t = global_state.callback_os_window->tabs + global_state.callback_os_window->active_tab;
                 for (window_idx = 0; window_idx < t->num_windows && t->windows[window_idx].id != w->id; window_idx++);
+                debug("sent to child as drag end\n");
                 handle_button_event(w, button, modifiers, window_idx);
                 clamp_to_window = false;
-                debug("sent to child as drag end\n");
                 return;
             }
         }
@@ -851,11 +874,10 @@ scroll_event(double UNUSED xoffset, double yoffset, int flags, int modifiers) {
     int s;
     bool is_high_resolution = flags & 1;
 
+// scale the scroll by the multiplier unless the mouse is grabbed. If the mouse is grabbed only change direction.
+#define SCALE_SCROLL(which) { double scale = OPT(which); if (screen->modes.mouse_tracking_mode) scale /= fabs(scale); yoffset *= scale; }
     if (is_high_resolution) {
-        yoffset *= OPT(touch_scroll_multiplier);
-        if (yoffset * screen->pending_scroll_pixels < 0) {
-            screen->pending_scroll_pixels = 0;  // change of direction
-        }
+        SCALE_SCROLL(touch_scroll_multiplier);
         double pixels = screen->pending_scroll_pixels + yoffset;
         if (fabs(pixels) < global_state.callback_os_window->fonts_data->cell_height) {
             screen->pending_scroll_pixels = pixels;
@@ -864,20 +886,20 @@ scroll_event(double UNUSED xoffset, double yoffset, int flags, int modifiers) {
         s = (int)round(pixels) / (int)global_state.callback_os_window->fonts_data->cell_height;
         screen->pending_scroll_pixels = pixels - s * (int) global_state.callback_os_window->fonts_data->cell_height;
     } else {
-        if (!screen->modes.mouse_tracking_mode) {
-            // Dont use multiplier if we are sending events to the application
-            yoffset *= OPT(wheel_scroll_multiplier);
-        } else if (OPT(wheel_scroll_multiplier) < 0) {
-            // ensure that changing scroll direction still works, even though
-            // we are not using wheel_scroll_multiplier
-            yoffset *= -1;
-        }
+        SCALE_SCROLL(wheel_scroll_multiplier);
         s = (int) round(yoffset);
-        // apparently on cocoa some mice generate really small yoffset values
-        // when scrolling slowly https://github.com/kovidgoyal/kitty/issues/1238
-        if (s == 0 && yoffset != 0) s = yoffset > 0 ? 1 : -1;
+        if (yoffset != 0) {
+            const int min_lines = screen->modes.mouse_tracking_mode ? 1 : OPT(wheel_scroll_min_lines);
+            if (min_lines > 0 && abs(s) < min_lines) s = yoffset > 0 ? min_lines : -min_lines;
+            // Always add the minimum number of lines when it is negative
+            else if (min_lines < 0) s = yoffset > 0 ? s - min_lines : s + min_lines;
+            // apparently on cocoa some mice generate really small yoffset values
+            // when scrolling slowly https://github.com/kovidgoyal/kitty/issues/1238
+            if (s == 0) s = yoffset > 0 ? 1 : -1;
+        }
         screen->pending_scroll_pixels = 0;
     }
+#undef SCALE_SCROLL
     if (s == 0) return;
     bool upwards = s > 0;
     if (screen->modes.mouse_tracking_mode) {
