@@ -411,6 +411,7 @@ dispatch_osc(Screen *screen, PyObject DUMP_UNUSED *dump_callback) {
             START_DISPATCH
             DISPATCH_OSC_WITH_CODE(set_color_table_color);
             END_DISPATCH
+        case 6:
         case 7:
             START_DISPATCH
             DISPATCH_OSC_WITH_CODE(process_cwd_notification);
@@ -1016,10 +1017,9 @@ dispatch_csi(Screen *screen, PyObject DUMP_UNUSED *dump_callback) {
 // DCS mode {{{
 
 static bool
-startswith(const uint32_t *string, size_t sz, const char *prefix) {
-    size_t l = strlen(prefix);
+startswith(const uint32_t *string, ssize_t sz, const char *prefix, ssize_t l) {
     if (sz < l) return false;
-    for (size_t i = 0; i < l; i++) {
+    for (ssize_t i = 0; i < l; i++) {
         if (string[i] != (unsigned char)prefix[i]) return false;
     }
     return true;
@@ -1062,29 +1062,36 @@ dispatch_dcs(Screen *screen, PyObject DUMP_UNUSED *dump_callback) {
             }
             break;
         case '@':
-#define CMD_PREFIX "kitty-cmd{"
-            if (startswith(screen->parser_buf + 1, screen->parser_buf_pos - 2, CMD_PREFIX)) {
-                PyObject *cmd = PyUnicode_FromKindAndData(PyUnicode_4BYTE_KIND, screen->parser_buf + 10, screen->parser_buf_pos - 10);
-                if (cmd != NULL) {
-                    REPORT_OSC2(screen_handle_cmd, (char)screen->parser_buf[0], cmd);
-                    screen_handle_cmd(screen, cmd);
-                    Py_DECREF(cmd);
-                } else PyErr_Clear();
-#undef CMD_PREFIX
-#define PRINT_PREFIX "kitty-print|"
-            } else if (startswith(screen->parser_buf + 1, screen->parser_buf_pos - 1, PRINT_PREFIX)) {
-                const size_t pp_size = sizeof(PRINT_PREFIX);
-                PyObject *msg = PyUnicode_FromKindAndData(PyUnicode_4BYTE_KIND, screen->parser_buf + pp_size, screen->parser_buf_pos - pp_size);
-                if (msg != NULL) {
-                    REPORT_OSC2(screen_handle_print, (char)screen->parser_buf[0], msg);
-                    screen_handle_print(screen, msg);
-                    Py_DECREF(msg);
-                } else PyErr_Clear();
-            } else {
-                REPORT_ERROR("Unrecognized DCS @ code: 0x%x", screen->parser_buf[1]);
+            if (startswith(screen->parser_buf + 1, screen->parser_buf_pos - 2, "kitty-", sizeof("kitty-") - 1)) {
+                if (startswith(screen->parser_buf + 7, screen->parser_buf_pos - 2, "cmd{", sizeof("cmd{") - 1)) {
+                    PyObject *cmd = PyUnicode_FromKindAndData(PyUnicode_4BYTE_KIND, screen->parser_buf + 10, screen->parser_buf_pos - 10);
+                    if (cmd != NULL) {
+                        REPORT_OSC2(screen_handle_cmd, (char)screen->parser_buf[0], cmd);
+                        screen_handle_cmd(screen, cmd);
+                        Py_DECREF(cmd);
+                    } else PyErr_Clear();
+#define IF_SIMPLE_PREFIX(prefix, func) \
+        if (startswith(screen->parser_buf + 7, screen->parser_buf_pos - 1, prefix, sizeof(prefix) - 1)) { \
+            const size_t pp_size = sizeof("kitty") + sizeof(prefix); \
+            PyObject *msg = PyUnicode_FromKindAndData(PyUnicode_4BYTE_KIND, screen->parser_buf + pp_size, screen->parser_buf_pos - pp_size); \
+            if (msg != NULL) { \
+                REPORT_OSC2(func, (char)screen->parser_buf[0], msg); \
+                screen_handle_kitty_dcs(screen, #func, msg); \
+                Py_DECREF(msg); \
+            } else PyErr_Clear();
+
+                } else IF_SIMPLE_PREFIX("overlay-ready|", handle_overlay_ready)
+                } else IF_SIMPLE_PREFIX("kitten-result|", handle_kitten_result)
+                } else IF_SIMPLE_PREFIX("print|", handle_remote_print)
+                } else IF_SIMPLE_PREFIX("echo|", handle_remote_echo)
+                } else IF_SIMPLE_PREFIX("ssh|", handle_remote_ssh)
+                } else IF_SIMPLE_PREFIX("ask|", handle_remote_askpass)
+#undef IF_SIMPLE_PREFIX
+                } else {
+                    REPORT_ERROR("Unrecognized DCS @ code: 0x%x", screen->parser_buf[1]);
+                }
             }
             break;
-#undef PRINT_PREFIX
         default:
             REPORT_ERROR("Unrecognized DCS code: 0x%x", screen->parser_buf[0]);
             break;
@@ -1177,7 +1184,7 @@ START_ALLOW_CASE_RANGE
 END_ALLOW_CASE_RANGE
             if (screen->parser_buf_pos > 0 && screen->parser_buf[screen->parser_buf_pos-1] == ESC) {
                 if (ch == '\\') { screen->parser_buf_pos--; return true; }
-                REPORT_ERROR("DCS sequence contained ESC without trailing \\ ignoring the sequence");
+                REPORT_ERROR("DCS sequence contained ESC without trailing \\ at pos: %u ignoring the sequence", screen->parser_buf_pos);
                 SET_STATE(ESC); return false;
             }
             if (screen->parser_buf_pos >= PARSER_BUF_SZ - 1) {
