@@ -24,6 +24,7 @@ typedef struct {
 typedef struct {
     monotonic_t visual_bell_duration, cursor_blink_interval, cursor_stop_blinking_after, mouse_hide_wait, click_interval;
     double wheel_scroll_multiplier, touch_scroll_multiplier;
+    int wheel_scroll_min_lines;
     bool enable_audio_bell;
     CursorShape cursor_shape;
     float cursor_beam_thickness;
@@ -80,6 +81,7 @@ typedef struct {
     struct {
         double outer, inner;
     } tab_bar_margin_height;
+    long macos_menubar_title_max_length;
 } Options;
 
 typedef struct WindowLogoRenderData {
@@ -104,6 +106,7 @@ typedef struct {
     monotonic_t at;
     int button, modifiers;
     double x, y;
+    unsigned long num;
 } Click;
 
 #define CLICK_QUEUE_SZ 3
@@ -298,12 +301,14 @@ typedef enum {
     NEXT_TAB,
     PREVIOUS_TAB,
     DETACH_TAB,
-    OPEN_FILE,
+    LAUNCH_URLS,
     NEW_WINDOW,
     CLOSE_WINDOW,
     RESET_TERMINAL,
+    CLEAR_TERMINAL_AND_SCROLLBACK,
     RELOAD_CONFIG,
     TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY,
+    TOGGLE_FULLSCREEN,
 
     NUM_COCOA_PENDING_ACTIONS
 } CocoaPendingAction;
@@ -337,5 +342,6 @@ uint8_t* draw_single_ascii_char(const char ch, size_t *result_width, size_t *res
 bool is_os_window_fullscreen(OSWindow *);
 void update_ime_focus(OSWindow* osw, bool focused);
 void update_ime_position(Window* w, Screen *screen);
-bool update_ime_position_for_window(id_type window_id, bool force, bool lost_focus);
+bool update_ime_position_for_window(id_type window_id, bool force, int update_focus);
 void set_ignore_os_keyboard_processing(bool enabled);
+void update_menu_bar_title(PyObject *title UNUSED);
